/***********************************************************************
 *
 * linux/arch/arm/mach-ns9360/core.c
 *
 * Copyright (C) 2004 by FS Forth-Systeme GmbH.start_kernelstart_kernel
 * All rights reserved.
 *
 * $Id: ns9360.h,v 1.5 2006-08-09 12:19:28 uzeisberger Exp $
 * @Author: Markus Pietrek
 * @References: [1] NS9750 Hardware Reference/December 2003
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 ***********************************************************************/
/***********************************************************************
 *  @History:
 *     2005/11/29 : moved to kernel 2.6.14
 *     2005/03/09 : inserted stuff from arch/arm/arch-ns9750/serial.h
 *                          for ST16C654 support
 *     2005/01/05 : adapted for NS9360
 *     2004/11/26 : USB Host added
 *     2004/08/20 : ported to kernel 2.6.8
 *     2004/08/17 : removed addr_*
 *     2004/07/20 : allocating BBUS on block
 *                  exporting ns_sys_clock_freq
 *     2004/06/16 : requesting resources
 ***********************************************************************/

#include <linux/init.h>		/* arch_initcall */
#include <linux/kernel.h>	/* ARRAY_SIZE */
#include <linux/ioport.h>	/* request_resource */
#include <linux/module.h>	/* EXPORT_SYMBOL */
#include <linux/device.h>
#include <linux/platform_device.h>

#include <asm/io.h>		/* writel */

#include <asm/mach-types.h>	/* MACH_TYPE_NS9360 */
#include <asm/mach/arch.h>	/* MACHINE_START */
#include <asm/mach/map.h>	/* map_desc */

#include <asm/arch/ns9360_config.h> /* NS_SYS_CLK_FREQ */
#include <asm/arch/ns9750_sys.h> /* NS_SYS_MISC */
#include <asm/arch/ns9750_bbus.h> /* ns_gpio_lock */
#include <asm/arch/ns9750_bbus_bridge.h> /* NS_BBUS_BRIDGE_MODULE_BASE */
#include <asm/arch/ns9750_mem.h>
#include <asm/arch/ns9360_usb.h>
#include <asm/arch/irqs.h>
#include <asm/arch/a9m9xx0dev.h>

#include <asm/arch/time.h> /* ns_time_init */


#include <asm/arch/a9m9xx0_ts.h>
#include <asm/arch/a9m9xx0_cf.h>

#include "devices.h"


static struct map_desc standard_io_desc[] __initdata = {
	/* virtual     physical    length      type */
	{ /* CS0 */
		.virtual	= 0xf4000000,
		.pfn		= __phys_to_pfn(0x40000000),
		.length		= 0x00001000,
		.type		= MT_DEVICE
	}, { /* CS2 */
		.virtual	= 0xf6000000,
		.pfn		= __phys_to_pfn(0x60000000),
		.length		= 0x000e0000,
		.type		= MT_DEVICE
#ifdef CONFIG_DIGI_WI_G
	}, { /* DIGI WI FPGA PROG */
		.virtual	= 0xf7000000,
		.pfn		= __phys_to_pfn(0x70000000),
		.length		= 0x000e0000,
		.type		= MT_DEVICE
#endif
	}, { /* BBus */
		.virtual	= 0xf9000000,
		.pfn		= __phys_to_pfn(0x90000000),
		.length		= 0x00900000,
		.type		= MT_DEVICE
	}, { /* AHB */
		.virtual	= 0xfa000000,
		.pfn		= __phys_to_pfn(0xa0000000),
		.length		= 0x00a00000,
		.type		= MT_DEVICE
	}
};

static struct resource resources[] = {
	{
		.name	= "bbus_utility",
		.start	= NS_BBUS_UTILITY_BASE,
		.end	= (NS_BBUS_UTILITY_BASE + NS_BBUS_ARM_WAKE_UP + 3)
	}, {
		.name	= "dma_0",
		.start	= (NS_BBUS_DMA_DESCR( 0, 0 )),
		.end	= (NS_BBUS_DMA_STAT( 0, 15 ) + 3)
	}, {
		.name	= "dma_1",
		.start	= (NS_BBUS_DMA_DESCR( 1, 0 )),
		.end	= (NS_BBUS_DMA_STAT( 1, 15 ) + 3)
	}, {
		.name = NULL
	}
};


static struct resource ns9360_usbhost_resources [] = {
        [0] = {
                .start  = (NS_USB_HCR),
                .end    = (NS_USB_HCR_PORT_SAT),
                .flags  = IORESOURCE_MEM,
        },
	[1] = {
		.start = (IRQ_USB_HOST_1),
		.end = (IRQ_USB_HOST_1),
		.flags = IORESOURCE_IRQ,
	},
};

static u64 ns9360_usbhost_dma_mask = 0xffffffffUL;

static struct platform_device ns9360_usbhost = {
        .name           = "ns9360-ohci",
        .id             = 0,
        .dev            = {
                .dma_mask = &ns9360_usbhost_dma_mask,
                .coherent_dma_mask = 0xffffffffUL,
        },
        .num_resources  = ARRAY_SIZE (ns9360_usbhost_resources),
        .resource       = ns9360_usbhost_resources,
};


static struct platform_device *usb_devices[] __initdata = {
        &ns9360_usbhost
};


static unsigned int _ns_sys_clock_freq;

spinlock_t ns_gpio_lock;	/* use it to lock all GPIO accesses */

extern void __init ns_init_irq( void );
static void __init ns_map_io( void );
void __init ns_init( void );

static void __init ns_map_io(void)
{
	/* called before ns_init */
	_ns_sys_clock_freq = NS_SYS_CLK_FREQ;

	/* @TODO */
	iotable_init( standard_io_desc, ARRAY_SIZE( standard_io_desc ) );

	printk( "NS9360 Rev. %i, running at %u MHz\n",
		(readl( NS_SYS_MISC ) >> 24) & 0xff,
		_ns_sys_clock_freq/2000000 );
}


/***********************************************************************
 * @Function:
 * @Return:
 * @Descr:  provides system clock for drivers
 ***********************************************************************/

unsigned int ns_sys_clock_freq( void )
{
	return _ns_sys_clock_freq;
}

EXPORT_SYMBOL( ns_sys_clock_freq );

