/*
 * arch/arm/mach-ns9xxx/dma.c
 *
 * Copyright (C) 2007 by Digi International Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 */

#include <linux/module.h>
#include <linux/dma-mapping.h>

#include <asm/io.h>
#include <asm/arch/regs-bbus.h>
#include <asm/arch/dma.h>


/***********************************************************************
 * @Function: ns9xxx_disable_dma_ch
 * @Return: nothing
 * @Descr: Disables the selected DMA channel for the selected DMA 
 *         controller.
 ***********************************************************************/
void ns9xxx_disable_dma_ch(int controller, int channel)
{
	u32 dma_ctrl;

	dma_ctrl = __raw_readl( addr_dma_ctrl(controller, channel) );
	dma_ctrl &= ~NS_BBUS_DMA_CTRL_CE;
	__raw_writel( dma_ctrl, addr_dma_ctrl(controller, channel) );
}

/***********************************************************************
 * @Function: ns9xxx_enable_dma_ch
 * @Return: nothing
 * @Descr: Enables the selected DMA channel for the selected DMA
 *         controller.
 ***********************************************************************/
void ns9xxx_enable_dma_ch(int controller, int channel)
{
	u32 dma_ctrl;

	dma_ctrl = __raw_readl( addr_dma_ctrl(controller, channel) );
	dma_ctrl |= NS_BBUS_DMA_CTRL_CE;
	__raw_writel( dma_ctrl, addr_dma_ctrl(controller, channel) );
}

/***********************************************************************
 * @Function: ns9xxx_set_options_dma_ch
 * @Return: nothing
 * @Descr: Configures the dma control, status and buffer descriptor 
 *         pointer with the function parameters.
 ***********************************************************************/
void ns9xxx_set_options_dma_ch(int controller, int channel,
			       dma_addr_t desc, u32 ctrl_opts, u32 stat_opts)
{
	__raw_writel( stat_opts, addr_dma_stat(controller, channel) );

	__raw_writel( (unsigned int)desc, 
	        addr_dma_bdesc(controller, channel) );
	
	__raw_writel( ctrl_opts,
	        addr_dma_ctrl(controller, channel) );
}

/***********************************************************************
 * @Function: ns9xxx_init_dma_desc_list
 * @Return: nothing
 * @Descr: Initializes the dma buffer descriptor list.
 ***********************************************************************/
void ns9xxx_init_dma_desc_list(ns_buffer_desc_t *first_desc,
			       u32 list_length, u32 enable_irq)
{
	int i;
	ns_buffer_desc_t *desc = first_desc;

	for (i = 1; i <= list_length; i++, desc++) {
		memset(desc, 0, sizeof(ns_buffer_desc_t));
		desc->s.bits.intr = enable_irq;
		desc->s.bits.wrap = (i != list_length) ? 0 : 1;
	}
}

EXPORT_SYMBOL(ns9xxx_disable_dma_ch);
EXPORT_SYMBOL(ns9xxx_enable_dma_ch);
EXPORT_SYMBOL(ns9xxx_set_options_dma_ch);
EXPORT_SYMBOL(ns9xxx_init_dma_desc_list);


