/*
 *  linux/arch/arm/mach-pxa/ccxp_pci.c
 *
 *  PCI functions for QuickLogic host PCI bridge
 *
 *  Copyright (C) 2004 QuickLogic Corp
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/pci.h>
#include <linux/ptrace.h>
#include <linux/slab.h>
#include <linux/ioport.h>
#include <linux/interrupt.h>
#include <linux/spinlock.h>
#include <linux/init.h>

#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/system.h>
#include <asm/mach/pci.h>
#include <asm/mach-types.h>

#include <asm/arch/pxa-regs.h>
#include <asm/arch/ccxp.h>
#include <asm/arch/irqs.h>

void *ql_pci_bridge_base;

/*#define FPGA_BASE_ADDR 0x0C000000*/
#define FPGA_BASE_ADDR 0x08000000
#define FPGA_REG_SIZE  0x100
 
#define PCI_ACCESS_READ  0
#define PCI_ACCESS_WRITE 1

static int irq_tab_quicklogic[4] __initdata = {
    CCXP_EXBRD_IRQ, CCXP_EXBRD_IRQ, CCXP_EXBRD_IRQ, CCXP_EXBRD_IRQ };

static struct resource pci_io_resource = {
	.name  = "pci io space",
	.start = CCXP_EXP_PCI_IO_BASE,
	.end   = CCXP_EXP_PCI_IO_BASE + CCXP_EXP_PCI_IO_SIZE -1,
	.flags = IORESOURCE_IO
};

static struct resource pci_mem_resource = {
	.name  = "pci memory space",
	.start = CCXP_EXP_PCI_MEM_BASE,
	.end   = CCXP_EXP_PCI_MEM_BASE + CCXP_EXP_PCI_MEM_SIZE -1,
	.flags = IORESOURCE_MEM
};


static spinlock_t quicklogic_pci_lock = SPIN_LOCK_UNLOCKED;

static int quicklogic_config_access(unsigned char access_type, struct pci_bus *bus, unsigned int dev_fn, unsigned char where, u32 * data)
{
        printk("PNDPND quicklogic_config_access\n");
	unsigned char busnum = bus->number;
	unsigned int device, function;
	unsigned long config_addr;
	unsigned int dummy, flags;	
        
        printk("PNDPND 1\n");
        /* We only have single PCI device attached */
        dummy = (dev_fn >> 3) & 0x1f;
        if ( dummy > 0 ) {
		*data = 0xffffffff;
                return -1;
        }

        printk("PNDPND 2\n");
        /* Since all IDSEL pins are pulled high, we don't really care bits[15:11] */
	if (busnum) {
        printk("PNDPND 3\n");
	    config_addr =  ((busnum & 0xff) << 0x10) | 
	                   ((dev_fn & 0xff) << 0x08) |
		  	   (where  & 0xfc) | 1;
	
	} else {			        
        printk("PNDPND 4\n");
            /* 7:3 = slot; 2:0 = function */
	    device = (dev_fn >> 3) & 0x1f;
            function = dev_fn & 0x7;
	
	    /* setup the configuration address */
	    /* Slot  Address Line */
	    /*  0  - AD11 */
	    /*  1  - AD12 */
	    /*  2  - AD13 */
	    /*  3  - AD14 */
	    /*  4  - AD15 */		
	    config_addr = CCXP_EXP_PCI_CONF_OFFSET | (function << 8) | (where & ~0x3);	
        printk("PNDPND function=%x\n", function);
        printk("PNDPND where=%x\n",where);
        }
       
        printk("PNDPND 5\n");
        spin_lock_irqsave(&quicklogic_pci_lock, flags); 

        printk("PNDPND 6\n");
	/* Force 32-bit read mode */
        dummy  = REG16(ql_pci_bridge_base + CCXP_EXP_GEN_CTRL);
        dummy &= ~GEN_CTRL_PCI_RD16_MODE; 
        REG16(ql_pci_bridge_base + CCXP_EXP_GEN_CTRL) = dummy;

        printk("PNDPND 7: config_addr=%x\n",config_addr);
        /* Set up PCI configuration address */
        /* bits[10:1] will be passed through */
        dummy = config_addr & 0x07FF;
	if (access_type == PCI_ACCESS_WRITE) {
        printk("PNDPND 7a\n");
		/* Always do 32-bits access */
		REG32(ql_pci_bridge_base + dummy) = *data;
        printk("PNDPND 7aa\n");
                spin_unlock_irqrestore(&quicklogic_pci_lock, flags);
        printk("PNDPND 7ab\n");
#if 1
                printk("Conf_Write: addr - 0x%04x data - 0x%08x\n", dummy, *data);
#endif
       } else {
        printk("PNDPND 7b: dummy=%x\n", dummy);
		/* Always do 32-bits access */
		*data  = REG32(ql_pci_bridge_base + dummy); 
        printk("PNDPND 7ba\n");
                spin_unlock_irqrestore(&quicklogic_pci_lock, flags);
        printk("PNDPND 7bb\n");
#if 1      
                printk("Conf_Read:  addr - 0x%04x data - 0x%08x\n", dummy, *data);
#endif
	}

        return 0; 
}

static int quicklogic_pcibios_read_config_byte(struct pci_bus *bus, unsigned int devfn, int where, u8 * val)
{
        printk("PNDPND quicklogic_pcibios_read_config_byte\n");
	u32 data;
	int ret;

	ret = quicklogic_config_access(PCI_ACCESS_READ, bus, devfn, where, &data);
        if (where & 1) data >>= 8;
        if (where & 2) data >>= 16;
        *val = data & 0xff;
	return ret;
}


static int quicklogic_pcibios_read_config_word(struct pci_bus *bus, unsigned int devfn, int where, u16 * val)
{
        printk("PNDPND quicklogic_pcibios_read_config_word\n");
	u32 data;
	int ret;

	ret = quicklogic_config_access(PCI_ACCESS_READ, bus, devfn, where, &data);
        if (where & 2) data >>= 16;
        *val = data & 0xffff;
	return ret;
}

static int quicklogic_pcibios_read_config_dword(struct pci_bus *bus, unsigned int devfn, int where, u32 * val)
{
        printk("PNDPND quicklogic_pcibios_read_config_dword\n");
	int ret;

	ret = quicklogic_config_access(PCI_ACCESS_READ, bus, devfn, where, val);
	return ret;
}


static int quicklogic_pcibios_write_config_byte(struct pci_bus *bus, unsigned int devfn, int where, u8 val)
{
        printk("PNDPND quicklogic_pcibios_write_config_byte\n");
	u32 data = 0;

	if (quicklogic_config_access(PCI_ACCESS_READ, bus, devfn, where, &data))
		return -1;

	data = (data & ~(0xff << ((where & 3) << 3))) |
	       (val << ((where & 3) << 3));

	if (quicklogic_config_access(PCI_ACCESS_WRITE, bus, devfn, where, &data))
		return -1;

	return PCIBIOS_SUCCESSFUL;
}

static int quicklogic_pcibios_write_config_word(struct pci_bus *bus, unsigned int devfn, int where, u16 val)
{
        printk("PNDPND quicklogic_pcibios_write_config_word\n");
        u32 data = 0;

	if (where & 1)
		return PCIBIOS_BAD_REGISTER_NUMBER;

        if (quicklogic_config_access(PCI_ACCESS_READ, bus, devfn, where, &data))
	       return -1;

	data = (data & ~(0xffff << ((where & 3) << 3))) |
	       (val << ((where & 3) << 3));

	if (quicklogic_config_access(PCI_ACCESS_WRITE, bus, devfn, where, &data))
	       return -1;


	return PCIBIOS_SUCCESSFUL;
}

static int quicklogic_pcibios_write_config_dword(struct pci_bus *bus, unsigned int devfn, int where, u32 val)
{
        printk("PNDPND quicklogic_pcibios_write_config_dword\n");
        u32 data = val;

	if (where & 3)
		return PCIBIOS_BAD_REGISTER_NUMBER;

	if (quicklogic_config_access(PCI_ACCESS_WRITE, bus, devfn, where, &data))
	       return -1;

	return PCIBIOS_SUCCESSFUL;
}

static int quicklogic_pcibios_read_config(struct pci_bus *bus, unsigned int devfn, int where, int size, u32 * val)
{
        printk("PNDPND quicklogic_pcibios_read_config\n");
        switch (size) {
           case 1:
                return quicklogic_pcibios_read_config_byte(bus, devfn, where, (u8 *)val); 
           case 2:
                return quicklogic_pcibios_read_config_word(bus, devfn, where, (u16 *)val); 
           default:
                return quicklogic_pcibios_read_config_dword(bus, devfn, where, (u32 *)val); 
        }

}

static int quicklogic_pcibios_write_config(struct pci_bus *bus, unsigned int devfn, int where, int size, u32 val)
{
        printk("PNDPND quicklogic_pcibios_write_config\n");
        switch (size) {
           case 1:
                return quicklogic_pcibios_write_config_byte(bus, devfn, where, (u8)val); 
           case 2:
                return quicklogic_pcibios_write_config_word(bus, devfn, where, (u16)val); 
           default:
                return quicklogic_pcibios_write_config_dword(bus, devfn, where, val); 
        }

}

struct pci_ops pci_quicklogic_ops = {
	.read  = quicklogic_pcibios_read_config,
	.write = quicklogic_pcibios_write_config,
};

static int __init pci_quicklogic_setup_resources(struct resource **resource)
{
        printk("PNDPND pci_quicklogic_setup_resources\n");
	if (request_resource(&iomem_resource, &pci_io_resource)) {
		printk(KERN_ERR "PCI: unable to allocate "
		       "io region\n");
		return -EBUSY;
	}
	if (request_resource(&iomem_resource, &pci_mem_resource)) {
		release_resource(&pci_io_resource);
		printk(KERN_ERR "PCI: unable to allocate "
		       "memory region\n");
		return -EBUSY;
	}

	/*
	 * bus->resource[0] is the IO resource for this bus
	 * bus->resource[1] is the mem resource for this bus
	 */
	resource[0] = &pci_io_resource;
	resource[1] = &pci_mem_resource;
	
	return 1;
}

int __init pci_quicklogic_setup(int nr, struct pci_sys_data *sys)
{
        printk("PNDPND pci_quicklogic_setup\n");
	int ret = 0;

	if (nr == 0) {
		sys->mem_offset = CCXP_EXP_PCI_MEM_BASE;
		ret = pci_quicklogic_setup_resources(sys->resource);
	}

	return ret;
}

struct pci_bus *pci_quicklogic_scan_bus(int nr, struct pci_sys_data *sys)
{
        printk("PNDPND pci_quicklogic_scan_bus\n");
	return pci_scan_bus(sys->busnr, &pci_quicklogic_ops, sys);
}


void __init pci_quicklogic_preinit(void)
{
        unsigned int dummy;
        
	/* set CS3 to be 16-bits VLIO *//* set CS5 to be 16-bits VLIO */
        MSC1   &= 0xFFFF;/*MSC2   &= 0xFFFF;*/
        MSC1   |= (0x7CCC << 16);/*MSC2   |= (0x7CCC << 16);*/
        dummy   = MSC1;/*dummy   = MSC2;*/
        printk("MSC1 Value: 0x%08x.\n", dummy);/*printk("MSC2 Value: 0x%08x.\n", dummy);*/
 
        /* enable GPIO CS3  */ 
        GPDR2  |= 0x8000;/*GPDR1  |= 0x0002;*/
        dummy   = GPDR2;/*dummy   = GPDR1;*/
        printk("GPDR2 Value: 0x%08x.\n", dummy);/*printk("GPDR1 Value: 0x%08x.\n", dummy);*/

        /* enable 13MHz Processor Oscillator Output  */ 
        OSCC  |= 0x08;
        dummy  = OSCC;
        printk("OSCC Value: 0x%08x.\n", dummy);
}

void __init pci_quicklogic_postinit(void)
{

}

/*
 * map the specified device/slot/pin to an IRQ.
 */
static int __init ccxp_map_irq(struct pci_dev *dev, u8 slot, u8 pin)
{
	int intnr = (slot + pin) & 3;

        return irq_tab_quicklogic[intnr];
}

static struct hw_pci ccxp_pci __initdata = {
	.swizzle          = NULL,
        .map_irq          = ccxp_map_irq,
        .setup            = pci_quicklogic_setup,
        .nr_controllers   = 1,
        .scan             = pci_quicklogic_scan_bus,
        .preinit          = pci_quicklogic_preinit,
        .postinit         = pci_quicklogic_postinit,
};

static int __init ccxp_pci_init(void)
{
        int ret = 0;
        void *fpga_base;
 
        if (!request_mem_region(CCXP_EXP_BASE, CCXP_EXP_SIZE, "PCI Bridge Registers"))
		printk(KERN_ERR "unable to reserve region\n"); 
        else {
        	ql_pci_bridge_base = ioremap(CCXP_EXP_BASE, CCXP_EXP_SIZE);
                if (!ql_pci_bridge_base) {
                        printk(KERN_ERR "unable to map region\n");
			ret = -ENOMEM;
                        goto out;
                }
        }
        printk("PCI Bridge Mapped Base Addr: 0x%08x\n", (u32)ql_pci_bridge_base);

        if (!request_mem_region(FPGA_BASE_ADDR, FPGA_REG_SIZE, "FPGA Board Registers"))
		printk(KERN_ERR "unable to reserve region\n"); 
        else {
        	fpga_base = ioremap(FPGA_BASE_ADDR, FPGA_REG_SIZE);
                if (!fpga_base) {
                        printk(KERN_ERR "unable to map region\n");
			ret = -ENOMEM;
                        goto out;
                }
        }
#if 0
        printk("MSCWR2:    0x%08x\n", *(volatile u32 *)(fpga_base + 0x84));
        printk("INTMSKEN:  0x%08x\n", *(volatile u32 *)(fpga_base + 0xC0));
        printk("INTSETCLR: 0x%08x\n", *(volatile u32 *)(fpga_base + 0xD0));
#endif

	if (machine_is_ccxp270())
               pci_common_init(&ccxp_pci);

#if 0
        printk("MSCWR2:    0x%08x\n", *(volatile u32 *)(fpga_base + 0x84));
        printk("INTMSKEN:  0x%08x\n", *(volatile u32 *)(fpga_base + 0xC0));
        printk("INTSETCLR: 0x%08x\n", *(volatile u32 *)(fpga_base + 0xD0));
#endif

out:
        iounmap(fpga_base);
        release_mem_region(FPGA_BASE_ADDR, FPGA_REG_SIZE); 
        return ret;
}

subsys_initcall(ccxp_pci_init);

EXPORT_SYMBOL(ql_pci_bridge_base); 
