/***********************************************************************
 *
 * linux/arch/arm/mach-s3c2410/mach-a9m2410.c
 *
 * Copyright (C) 2005 by FS Forth-Systeme GmbH
 * All rights reserved.
 *
 * $Id: mach-a9m2410.c,v 1.8 2005-11-30 13:06:52 jjaeger Exp $
 * @Author: Jonas Dietsche, Joachim Jaeger
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 * @History:
 * 30-03-2005:	adapted to A9M2410 module		Joachim Jaeger
 * 30-03-2005:	file copied from mach-smdk2410.c	Joachim Jaeger
 * derived from linux/arch/arm/mach-s3c2410/mach-bast.c, written by
 * Ben Dooks <ben@simtec.co.uk>
 ***********************************************************************/

#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/interrupt.h>
#include <linux/list.h>
#include <linux/timer.h>
#include <linux/init.h>
#include <linux/device.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/irq.h>

#include <asm/hardware.h>
#include <asm/hardware/iomd.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/mach-types.h>

#include <linux/serial_core.h>
#include <linux/fb.h>

#include <asm/arch/regs-serial.h>
#include <asm/arch/regs-gpio.h>
#include <asm/arch/fb.h>
#include <asm/arch/a9m24x0.h>
#include <asm/arch/s3c2410_ts.h>

#include "s3c2410.h"
#include "clock.h"
#include "devs.h"
#include "cpu.h"
#include "a9m24x0_nand.h"
#include "pm.h"

static struct map_desc a9m24x0_iodesc[] __initdata = {
#ifdef CONFIG_A9M24XX_QUART
	/* Map the external quad UART */
	{A9M24X0_QUART_IO, 0x11000000, SZ_1M, MT_DEVICE},
#endif /* CONFIG_A9M24XX_QUART */
	/* Map the ethernet controller CS8900A */
	{A9M24X0_ETH_IO, 0x29000000, SZ_1M, MT_DEVICE}
};

#define A9M24X0_WIDTH		320
#define A9M24X0_HEIGHT		240

#define PHIL_LB121_WIDTH 	800
#define PHIL_LB121_HEIGHT 	600

#define HIT_SX14Q005_WIDTH	320
#define HIT_SX14Q005_HEIGHT	240

#define OPT_T51750_WIDTH	640
#define OPT_T51750_HEIGHT	480

#define UCON S3C2410_UCON_DEFAULT
#define ULCON S3C2410_LCON_CS8 | S3C2410_LCON_PNONE | S3C2410_LCON_STOPB
#define UFCON S3C2410_UFCON_RXTRIG8 | S3C2410_UFCON_FIFOMODE

#ifdef CONFIG_A9M24XX_QUART
#include <linux/serial_8250.h>
#include <asm/arch/irqs.h>

#define ST16C654_BASE_UARTA (A9M24X0_QUART_IO)		/* UART1 CS2 */
#define ST16C654_BASE_UARTB (A9M24X0_QUART_IO + 0x08)	/* UART2 CS2 */
#define ST16C654_BASE_UARTC (A9M24X0_QUART_IO + 0x10)	/* UART3 CS2 */
#define ST16C654_BASE_UARTD (A9M24X0_QUART_IO + 0x18)	/* UART4 CS2 */

static struct plat_serial8250_port serial_platform_data[] = {
	{
		.iobase		= ST16C654_BASE_UARTA,
		.membase	= (unsigned char*)ST16C654_BASE_UARTA,
		.mapbase	= ST16C654_BASE_UARTA,
		.irq		= A9M24X0_QUART_IRQ,
		.iotype		= UPIO_MEM,
		.uartclk	= 1843200,
		.regshift	= 0,
		.flags		= UPF_BOOT_AUTOCONF,
	},
	{
		.iobase		= ST16C654_BASE_UARTB,
		.membase	= (unsigned char*)ST16C654_BASE_UARTB,
		.mapbase	= ST16C654_BASE_UARTB,
		.irq		= A9M24X0_QUART_IRQ,
		.iotype		= UPIO_MEM,
		.uartclk	= 1843200,
		.regshift	= 0,
		.flags		= UPF_BOOT_AUTOCONF,
	},
	{
		.iobase		= ST16C654_BASE_UARTC,
		.membase	= (unsigned char*)ST16C654_BASE_UARTC,
		.mapbase	= ST16C654_BASE_UARTC,
		.irq		= A9M24X0_QUART_IRQ,
		.iotype		= UPIO_MEM,
		.uartclk	= 1843200,
		.regshift	= 0,
		.flags		= UPF_BOOT_AUTOCONF,
	},
	{
		.iobase		= ST16C654_BASE_UARTD,
		.membase	= (unsigned char*)ST16C654_BASE_UARTD,
		.mapbase	= ST16C654_BASE_UARTD,
		.irq		= A9M24X0_QUART_IRQ,
		.iotype		= UPIO_MEM,
		.uartclk	= 1843200,
		.regshift	= 0,
		.flags		= UPF_BOOT_AUTOCONF,
	},
	{ },
};

static struct platform_device serial_device = {
	.name			= "serial8250",
	.id			= 0,
	.dev			= {
		.platform_data	= serial_platform_data,
	},
};
#endif /*CONFIG_A9M24XX_QUART*/

static struct s3c2410_uartcfg a9m24x0_uartcfgs[] = {
	[0] = {
		.hwport	     = 0,
		.flags	     = 0,
		.ucon	     = UCON,
		.ulcon	     = ULCON,
		.ufcon	     = UFCON,
	},
	[1] = {
		.hwport	     = 1,
		.flags	     = 0,
		.ucon	     = UCON,
		.ulcon	     = ULCON,
		.ufcon	     = UFCON,
	},
	[2] = {
		.hwport	     = 2,
		.flags	     = 0,
		.ucon	     = UCON,
		.ulcon	     = ULCON,
		.ufcon	     = UFCON,
	}
};

/**
 * Set lcd on or off
 **/
static void h1940_lcd_power(int on)
{
#if 0
	s3c2410_gpio_setpin(S3C2410_GPC0, on);
#endif
}

/**
 * Set the backlight on or off.
 **/
static void h1940_backlight_power(int on)
{
	/* use LCD_PWREN feature */
	writel( readl( S3C2410_GPGCON ) | (3 << 8 ), S3C2410_GPGCON );

        if (!on)
		/* backlight on */
		writel( readl( S3C2410_GPGDAT ) | ~(1 << 4 ), S3C2410_GPGDAT );
	else
		/* backlight off */
		writel( readl( S3C2410_GPGDAT ) | (1 << 4 ), S3C2410_GPGDAT );
}

/* Set the backlight pwm according to the level from wince */

#include <asm/arch/regs-timer.h>
static void h1940_set_brightness(int level)
{
#if 0
	unsigned long tcfg0;
	unsigned long tcfg1;
	unsigned long tcmpb0=0x00;
	unsigned long tcon;

	/* configure power on/off */
	h1940_backlight_power(level ? 1 : 0);

	switch (level)
	{
		case 0:
			break;
		case 1:
			tcmpb0=0x0b;
			break;
		default:
		case 2:
			tcmpb0=0x16;
			break;
		case 3:
			tcmpb0=0x21;
			break;
		case 4:
			tcmpb0=0x2c;
			break;
	}

	tcfg0=__raw_readl(S3C2410_TCFG0);
	tcfg1=__raw_readl(S3C2410_TCFG1);

	tcfg0 &= ~S3C2410_TCFG_PRESCALER0_MASK;
	tcfg0 |= 0x18;

	tcfg1 &= ~S3C2410_TCFG1_MUX0_MASK;
	tcfg1 |= S3C2410_TCFG1_MUX0_DIV2;

	__raw_writel(tcfg0, S3C2410_TCFG0);
	__raw_writel(tcfg1, S3C2410_TCFG1);
	__raw_writel(0x31, S3C2410_TCNTB(0));

	tcon = __raw_readl(S3C2410_TCON);
	tcon &= ~0x0F;
	tcon |= S3C2410_TCON_T0RELOAD;
	tcon |= S3C2410_TCON_T0MANUALUPD;

	__raw_writel(tcon, S3C2410_TCON);
	__raw_writel(0x31, S3C2410_TCNTB(0));
	__raw_writel(tcmpb0, S3C2410_TCMPB(0));

	/* start the timer running */
	tcon |= S3C2410_TCON_T0START;
	tcon &= ~S3C2410_TCON_T0MANUALUPD;
	__raw_writel(tcon, S3C2410_TCON);
#endif
}

#include <asm/arch/regs-lcd.h>

struct s3c2410fb_mach_info a9m2410fb_info[] = {
	{
	.fixed_syncs=		1,
	.regs={
		.lcdcon1=	S3C2410_LCDCON1_TFT16BPP | \
				S3C2410_LCDCON1_TFT | \
		 		S3C2410_LCDCON1_CLKVAL(7), /* 6.3 MHz */

		.lcdcon2=	S3C2410_LCDCON2_VBPD(5) | \
				S3C2410_LCDCON2_LINEVAL(A9M24X0_HEIGHT-1) | \
				S3C2410_LCDCON2_VFPD(5) | \
				S3C2410_LCDCON2_VSPW(0),

		.lcdcon3=	S3C2410_LCDCON3_HBPD(9) | \
				S3C2410_LCDCON3_HOZVAL(A9M24X0_WIDTH-1) | \
				S3C2410_LCDCON3_HFPD(10),

		.lcdcon4=	S3C2410_LCDCON4_MVAL(0) | \
				S3C2410_LCDCON4_HSPW(1),

		.lcdcon5=	S3C2410_LCDCON5_FRM565 | \
				S3C2410_LCDCON5_INVVLINE | \
				S3C2410_LCDCON5_INVVFRAME |
				S3C2410_LCDCON5_INVPWREN |
				S3C2410_LCDCON5_PWREN |
				S3C2410_LCDCON5_HWSWP,
	},
	.lpcsel=	0x04,
	.gpccon=	0xAAAA56AA,
	.gpccon_mask=	0xffffffff,
	.gpcdat=	0x00000040,
	.gpcdat_mask=	0xffffffff,
	.gpcup=		0x0000ffff,
	.gpcup_mask=	0xffffffff,
	.gpdcon=	0xaaa0aaa0,
	.gpdcon_mask=	0xffffffff,
	.gpdup=		0x0000ffff,
	.gpdup_mask=	0xffffffff,

	.width=		A9M24X0_WIDTH,
	.height=	A9M24X0_HEIGHT,
	.xres=		{A9M24X0_WIDTH,A9M24X0_WIDTH,A9M24X0_WIDTH},
	.yres=		{A9M24X0_HEIGHT,A9M24X0_HEIGHT,A9M24X0_HEIGHT},
	.bpp=		{16,16,16},

	.backlight_min		= 0,
	.backlight_max		= 4,
	.backlight_default	= 2,

	.lcd_power		= h1940_lcd_power,
	.backlight_power	= h1940_backlight_power,
	.set_brightness		= h1940_set_brightness,

	.name= 		"a9m2410",
	.cfg_gpio= 	1,
	.visual= 	FB_VISUAL_TRUECOLOR,

	.red_offset=	11,
	.green_offset=	5,
	.blue_offset=	0,
	.transp_offset=	0,
	.red_length=	5,
	.green_length=	6,
	.blue_length=	5,
	.transp_length=	0,

	/* LUT's taken from lcdlib.c example of Samsung */
	.lut=	{ (0xfdb96420) /* red*/,
		  (0xfdb96420) /* green */,
		  (0x0000fb40) /* blue */ },
	}, {
	.fixed_syncs=		1,
	.regs={
		.lcdcon1=	S3C2410_LCDCON1_TFT16BPP | \
				S3C2410_LCDCON1_TFT | \
			 	S3C2410_LCDCON1_CLKVAL(1), /* 25.35 MHz < 40MHz max. display */

		.lcdcon2=	S3C2410_LCDCON2_VBPD(5) | \
				S3C2410_LCDCON2_LINEVAL(PHIL_LB121_HEIGHT-1) | \
				S3C2410_LCDCON2_VFPD(1),

		.lcdcon3=	S3C2410_LCDCON3_HBPD(12) | \
				S3C2410_LCDCON3_HOZVAL(PHIL_LB121_WIDTH-1) | \
				S3C2410_LCDCON3_HFPD(12),

		.lcdcon4=	S3C2410_LCDCON4_HSPW(1),

		.lcdcon5=	S3C2410_LCDCON5_FRM565 | \
				S3C2410_LCDCON5_INVVLINE | \
				S3C2410_LCDCON5_INVVFRAME |
/*				S3C2410_LCDCON5_INVPWREN |
				S3C2410_LCDCON5_PWREN |*/
				S3C2410_LCDCON5_HWSWP,
	},
	.lpcsel=	0x04,
	.gpccon=	0xAAAA56AA,
	.gpccon_mask=	0xffffffff,
	.gpcdat=	0x00000040,
	.gpcdat_mask=	0xffffffff,
	.gpcup=		0x0000ffff,
	.gpcup_mask=	0xffffffff,
	.gpdcon=	0xaaa0aaa0,
	.gpdcon_mask=	0xffffffff,
	.gpdup=		0x0000ffff,
	.gpdup_mask=	0xffffffff,

	.width=		PHIL_LB121_WIDTH,
	.height=	PHIL_LB121_HEIGHT,
	.xres=		{PHIL_LB121_WIDTH,PHIL_LB121_WIDTH,PHIL_LB121_WIDTH},
	.yres=		{PHIL_LB121_HEIGHT,PHIL_LB121_HEIGHT,PHIL_LB121_HEIGHT},
	.bpp=		{16,16,16},

	.backlight_min		= 0,
	.backlight_max		= 4,
	.backlight_default	= 2,

	.lcd_power		= h1940_lcd_power,
	.backlight_power	= h1940_backlight_power,
	.set_brightness		= h1940_set_brightness,

	.name= 		"phil_lb121",
	.cfg_gpio= 	1,
	.visual= 	FB_VISUAL_TRUECOLOR,

	.red_offset=	11,
	.green_offset=	5,
	.blue_offset=	0,
	.transp_offset=	0,
	.red_length=	5,
	.green_length=	6,
	.blue_length=	5,
	.transp_length=	0,

	/* LUT's taken from lcdlib.c example of Samsung */
	.lut=	{ (0xfdb96420) /* red*/,
		  (0xfdb96420) /* green */,
		  (0x0000fb40) /* blue */ },

	}, {
	.fixed_syncs=		1,
	.regs={
		.lcdcon1=	S3C2410_LCDCON1_STN8 |
				S3C2410_LCDCON1_STN8BPP |
				/* 433ns CL2, 52.4us CL1, 79.5 Hz VFRAME, CL2/ */
				S3C2410_LCDCON1_CLKVAL( 22 ),

		.lcdcon2=	S3C2410_LCDCON2_LINEVAL(HIT_SX14Q005_HEIGHT-1 ),

		.lcdcon3=	S3C2410_LCDCON3_WDLY( 0 ) | /* at least 50ns,
								we have ~160ns */
				S3C2410_LCDCON3_HOZVAL( ((HIT_SX14Q005_WIDTH*3)/8)-1 ) |
				S3C2410_LCDCON3_LINEBLANK( 16 ),

		.lcdcon4=	S3C2410_LCDCON4_WLH( 0 ), /* at least 80ns, we have
						     * ~160ns */

		.lcdcon5=	S3C2410_LCDCON5_BSWP |
				S3C2410_LCDCON5_PWREN,
	},
	.lpcsel=	0x04,
	.gpccon=	0xAAAA56AA,
	.gpccon_mask=	0xffffffff,
	.gpcdat=	0x00000040,
	.gpcdat_mask=	0xffffffff,
	.gpcup=		0x0000ffff,
	.gpcup_mask=	0xffffffff,
	.gpdcon=	0xaaa0aaa0,
	.gpdcon_mask=	0xffffffff,
	.gpdup=		0x0000ffff,
	.gpdup_mask=	0xffffffff,

	.width=		HIT_SX14Q005_WIDTH,
	.height=	HIT_SX14Q005_HEIGHT,
	.xres=		{HIT_SX14Q005_WIDTH,HIT_SX14Q005_WIDTH,HIT_SX14Q005_WIDTH},
	.yres=		{HIT_SX14Q005_HEIGHT,HIT_SX14Q005_HEIGHT,HIT_SX14Q005_HEIGHT},
	.bpp=		{8,8,8},

	.backlight_min		= 0,
	.backlight_max		= 4,
	.backlight_default	= 2,

	.lcd_power		= h1940_lcd_power,
	.backlight_power	= h1940_backlight_power,
	.set_brightness		= h1940_set_brightness,

	.name= 		"hit_sx14q005",
	.cfg_gpio= 	1,
	.visual= 	FB_VISUAL_TRUECOLOR,

	.red_offset=	5,
	.green_offset=	2,
	.blue_offset=	0,
	.transp_offset=	0,
	.red_length=	3,
	.green_length=	3,
	.blue_length=	2,
	.transp_length=	0,

	/* LUT's taken from lcdlib.c example of Samsung */
	.lut=	{ (0xfdb96420) /* red*/,
		  (0xfdb96420) /* green */,
		  (0x0000fb40) /* blue */ },

	}, {
	.fixed_syncs=		1,
	.regs={
		.lcdcon1=	S3C2410_LCDCON1_TFT16BPP | \
				S3C2410_LCDCON1_TFT | \
			 	S3C2410_LCDCON1_CLKVAL(1), /* 25.35 MHz */

		.lcdcon2=	S3C2410_LCDCON2_VBPD(17) | \
				S3C2410_LCDCON2_LINEVAL(OPT_T51750_HEIGHT-1) | \
				S3C2410_LCDCON2_VFPD(20) | \
				S3C2410_LCDCON2_VSPW(3),

		.lcdcon3=	S3C2410_LCDCON3_HBPD(2) | \
				S3C2410_LCDCON3_HOZVAL(OPT_T51750_WIDTH-1) | \
				S3C2410_LCDCON3_HFPD(0),

		.lcdcon4=	S3C2410_LCDCON4_MVAL(0) | \
				S3C2410_LCDCON4_HSPW(5),

		.lcdcon5=	S3C2410_LCDCON5_FRM565 | \
				S3C2410_LCDCON5_INVVLINE | \
				S3C2410_LCDCON5_INVVFRAME |
				S3C2410_LCDCON5_INVPWREN |
				S3C2410_LCDCON5_PWREN |
				S3C2410_LCDCON5_HWSWP,
	},
	.lpcsel=	0x04,
	.gpccon=	0xAAAA56AA,
	.gpccon_mask=	0xffffffff,
	.gpcdat=	0x00000040,
	.gpcdat_mask=	0xffffffff,
	.gpcup=		0x0000ffff,
	.gpcup_mask=	0xffffffff,
	.gpdcon=	0xaaa0aaa0,
	.gpdcon_mask=	0xffffffff,
	.gpdup=		0x0000ffff,
	.gpdup_mask=	0xffffffff,

	.width=		OPT_T51750_WIDTH,
	.height=	OPT_T51750_HEIGHT,
	.xres=		{OPT_T51750_WIDTH,OPT_T51750_WIDTH,OPT_T51750_WIDTH},
	.yres=		{OPT_T51750_HEIGHT,OPT_T51750_HEIGHT,OPT_T51750_HEIGHT},
	.bpp=		{16,16,16},

	.backlight_min		= 0,
	.backlight_max		= 4,
	.backlight_default	= 2,

	.lcd_power		= h1940_lcd_power,
	.backlight_power	= h1940_backlight_power,
	.set_brightness		= h1940_set_brightness,

	.name= 		"opt_t51750",
	.cfg_gpio= 	1,
	.visual= 	FB_VISUAL_TRUECOLOR,

	.red_offset=	11,
	.green_offset=	5,
	.blue_offset=	0,
	.transp_offset=	0,
	.red_length=	5,
	.green_length=	6,
	.blue_length=	5,
	.transp_length=	0,

	/* LUT's taken from lcdlib.c example of Samsung */
	.lut=	{ (0xfdb96420) /* red*/,
		  (0xfdb96420) /* green */,
		  (0x0000fb40) /* blue */ },
	}
};

static struct s3c2410_ts_mach_info a9m24x0_ts_cfg __initdata = {
		.delay = 10000,
		.presc = 49,
		.oversampling_shift = 2,
};

static struct platform_device *a9m2410_devices[] __initdata = {
	&s3c_device_usb,
	&s3c_device_lcd,
	&s3c_device_wdt,
	&s3c_device_i2c,
	&s3c_device_iis,
	&s3c_device_nand,
	&s3c_device_sdi,
	&s3c_device_usbgadget,
	&s3c_device_ts,
	&s3c_device_spi0,
};

static struct s3c24xx_board a9m24x0_board __initdata = {
	.devices       = a9m2410_devices,
	.devices_count = ARRAY_SIZE(a9m2410_devices)
};

static void __init a9m24x0_map_io(void)
{
	s3c24xx_init_io(a9m24x0_iodesc, ARRAY_SIZE(a9m24x0_iodesc));
	s3c24xx_init_clocks(0);
	s3c24xx_init_uarts(a9m24x0_uartcfgs, ARRAY_SIZE(a9m24x0_uartcfgs));
	s3c24xx_set_board(&a9m24x0_board);
}

static void __init a9m24x0_init_irq(void)
{
	s3c24xx_init_irq();
}

static void __init a9m24x0_init(void)
{
	set_s3c2410ts_info(&a9m24x0_ts_cfg);
	set_s3c2410fb_info(&a9m2410fb_info[0]);
	a9m24x0_init_nand(&s3c_device_nand.dev);
#ifdef CONFIG_A9M24XX_QUART
	platform_device_register(&serial_device);
#endif /* CONFIG_A9M24XX_QUART */
/*	s3c2410_pm_init(); TODO: check all drivers before using PM*/
}

MACHINE_START(A9M2410, "A9M2410")
	/* Maintainer: Joachim Jaeger <joachim_jaeger@digi.com> */
	.phys_ram	= S3C2410_SDRAM_PA,
	.phys_io	= S3C2410_PA_UART,
	.io_pg_offst	= (((u32)S3C24XX_VA_UART) >> 18) & 0xfffc,
	.boot_params	= S3C2410_SDRAM_PA + 0x100,
	.map_io		= a9m24x0_map_io,
	.init_irq	= a9m24x0_init_irq,
	.init_machine	= a9m24x0_init,
	.timer		= &s3c24xx_timer,
MACHINE_END


