/***********************************************************************
 *
 *  Copyright 2005 by Sistemas Embebidos S.A.  
 *  A Digi International Company.
 *  All rights reserved.
 *
 *  $Id:
 *  @Author: Neil Bryan
 *  @Descr:  Low level hardware interface to the AT91RM9200 PIO to allow
 *           for the use of the algo bit I2C Bit Banging driver.
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
 *  
 ***********************************************************************
 *  @History:
 *     2005/11/16 : NB : Initial version.
 *
 ***********************************************************************/

#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/i2c.h>
#include <linux/i2c-algo-bit.h>
#include <asm/hardware.h>
#include <asm/arch/pio.h>

/* GPIO bit mask */
#define DATA_MASK 	AT91_PIN_PA25	
#define CLOCK_MASK 	AT91_PIN_PA26	

#ifndef AT91_I2C_BB_UDELAY
# define AT91_I2C_BB_UDELAY	1
#endif

static inline void at91_i2c_bb_set_sda( void *data, int state )
{
#if 0
	/* configure DATA as output  */
	AT91_SYS->PIOA_OER = DATA_MASK;

	if( state )
		AT91_SYS->PIOA_SODR = DATA_MASK;
	else
		AT91_SYS->PIOA_CODR = DATA_MASK;
#endif
	at91_set_gpio_output( DATA_MASK, 0 );
	at91_set_gpio_value( DATA_MASK, state );
}

static inline void at91_i2c_bb_set_scl( void *data, int state )
{
#if 0
	if( state )
		AT91_SYS->PIOA_SODR = CLOCK_MASK;
	else
		AT91_SYS->PIOA_CODR = CLOCK_MASK;
#endif
	at91_set_gpio_value( CLOCK_MASK, state );

}

static inline int at91_i2c_bb_get_sda( void *data )
{
#if 0
	/* Configure DATA as input  */
	AT91_SYS->PIOA_ODR = DATA_MASK;

	return( AT91_SYS->PIOA_PDSR & DATA_MASK );
#endif
	at91_set_gpio_input( DATA_MASK, 0 );
	return at91_get_gpio_value( DATA_MASK );

}

static inline int at91_i2c_bb_get_scl( void *data )
{
//	return( AT91_SYS->PIOA_PDSR & CLOCK_MASK );
	return at91_get_gpio_value( CLOCK_MASK );
}

static struct i2c_algo_bit_data at91_i2c_bb_data = 
{
	.setsda		= at91_i2c_bb_set_sda,
	.setscl		= at91_i2c_bb_set_scl,
	.getsda		= at91_i2c_bb_get_sda,
	.getscl		= at91_i2c_bb_get_scl,
	.udelay		= AT91_I2C_BB_UDELAY,
	.mdelay		= 80,
	.timeout	= HZ
};

static struct i2c_adapter at91_i2c_bb_ops = 
{
	.owner		= THIS_MODULE,
	.id		= I2C_HW_B_AT91,
	.algo_data	= &at91_i2c_bb_data,
	.name		= "AT91 bit-banging algo i2c driver"
};

static int __init at91_i2c_bb_init( void )
{
	printk( KERN_ERR "AT91RM9200 Bit-Banging I2C driver\n" );
#if 0         
	/* Activate PIO mode  */
        AT91_SYS->PIOA_PER = AT91C_PIO_PA25 | AT91C_PIO_PA26;
        /* Configure as output */
        AT91_SYS->PIOA_OER = AT91C_PIO_PA25 | AT91C_PIO_PA26;
	/* Set outputs */
	AT91_SYS->PIOA_SODR = AT91C_PIO_PA25 | AT91C_PIO_PA26;
	
	/* Enable peripheral clock  */
	AT91_SYS->PMC_PCER = 1 << AT91C_ID_PIOA;
#endif 
	at91_set_gpio_output( DATA_MASK, 0 );
	at91_set_gpio_value( DATA_MASK, 1 );
	at91_set_gpio_output( CLOCK_MASK, 0 );
	at91_set_gpio_value( CLOCK_MASK, 1 );

	/* @TODO verifz if following is needed  */
	//AT91_SYS->PMC_PCER = 1 << AT91C_ID_PIOA;

	return i2c_bit_add_bus( &at91_i2c_bb_ops );
}

static void __exit at91_i2c_bb_exit( void )
{
	i2c_bit_del_bus( &at91_i2c_bb_ops );
}

MODULE_AUTHOR( "Neil Bryan <neil.bryan@digi.com>" );
MODULE_DESCRIPTION( "I2C-Bus Bit Banging adapter for AT91" );
MODULE_LICENSE( "GPL" );

module_init( at91_i2c_bb_init );
module_exit( at91_i2c_bb_exit );

