#include <linux/module.h>
#include <linux/config.h>
#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/device.h>
#include <linux/dma-mapping.h>
#include <asm/uaccess.h>
#include <asm/dma.h>

#include <linux/spi.h>

#include "fsalloc.h"

void *fs_memory_alloc(size_t size, int base)
{
	spi_fs_msg_buff_t *buff;

	buff = (spi_fs_msg_buff_t *) kmalloc(sizeof(spi_fs_msg_buff_t), base);
	buff->size = size;
	buff->io_buffer =
	    dma_alloc_coherent(NULL, size, &buff->dma_buffer, base);

	pr_debug
	    ("%s:allocated memory(%p) for io_buffer = %p dma_buffer = %08x\n",
	     __FUNCTION__, buff, buff->io_buffer, buff->dma_buffer);

	return (void *)buff;
}

void fs_memory_free(const void *data)
{
	spi_fs_msg_buff_t *buff;

	buff = (spi_fs_msg_buff_t *) data;
	pr_debug("%s:deleted memory(%p) for io_buffer = %p dma_buffer = %08x\n",
		 __FUNCTION__, buff, buff->io_buffer, buff->dma_buffer);

	dma_free_coherent(NULL, buff->size, buff->io_buffer, buff->dma_buffer);
	kfree(buff);
}

void* fs_get_buffer( struct device *device, void* void_buff ) 
{
	spi_fs_msg_buff_t *buff = void_buff;
	return buff->io_buffer;
}

void fs_release_buffer( struct device* dev, void* buffer )
{
}

EXPORT_SYMBOL(fs_get_buffer);
EXPORT_SYMBOL(fs_release_buffer);
EXPORT_SYMBOL(fs_memory_alloc);
EXPORT_SYMBOL(fs_memory_free);
