/*
 * drivers/usb/gadget/ns9xxx_udc.h
 *
 * Copyright (C) 2006 Sascha Hauer <s.hauer@pengutronix.de>, Pengutronix
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#define BBUS_DMA_CHANNELS	16
#define BBUS_DMA_MAX_DESC	8

/* beware: unlike the datasheet we count the DMA channels from zero */

/* DMA Buffer descriptor Pointer */
#define BBUS_DMA_BUF_DESC(chan)	((chan) * 0x20)

/* DMA Control register */
#define BBUS_DMA_BUF_CR(chan)	((chan) * 0x20 + 0x10)
#define BBUS_DMA_CR_CE		(1<<31)
#define BBUS_DMA_CR_CA		(1<<30)
#define BBUS_DMA_CR_MODE_BTE4	(2<<24)
#define BBUS_DMA_CR_FLY_BY_READ	(1<<26)
#define BBUS_DMA_CR_BDR		(1<<22)
#define BBUS_DMA_CR_RESET	(1<<18)

/* DMA Status/Interrupt Enable register */
#define BBUS_DMA_SIER(chan)	((chan) * 0x20 + 0x14)
#define BBUS_DMA_SIER_NCIP	(1<<31)
#define BBUS_DMA_SIER_ECIP	(1<<30)
#define BBUS_DMA_SIER_NRIP	(1<<29)
#define BBUS_DMA_SIER_CAIP	(1<<28)
#define BBUS_DMA_SIER_PCIP	(1<<27)
#define BBUS_DMA_SIER_NCIE	(1<<24)
#define BBUS_DMA_SIER_ECIE	(1<<23)
#define BBUS_DMA_SIER_NRIE	(1<<22)
#define BBUS_DMA_SIER_CAIE	(1<<21)
#define BBUS_DMA_SIER_PCIE	(1<<20)

#define EP_FIFO_SIZE		32

#define DMA_ADDR_INVALID	(~(dma_addr_t)0)

struct bbus_dma_desc {
	unsigned long sadr;
	unsigned long buflen;
	unsigned long dadr;
	unsigned long flags;
};

#define BBUS_DMA_WRAP		(1<<31)
#define BBUS_DMA_INTERRUPT	(1<<30)
#define BBUS_DMA_LAST		(1<<29)
#define BBUS_DMA_FULL		(1<<28)
#define BBUS_DMA_SUCCESS	(1<<13)
#define BBUS_DMA_SETUP_CMD	(1<<12)

/* UDFE Control and Status register 0 */
#define UDFE_CTRL0		0x00
#define CTRL0_WKUP		(1<<2)
#define CTRL0_SUSP		(1<<5)
#define CTRL0_DRST		(3<<9)

/* UDFE Control and Status register 1 */
#define UDFE_CTRL1		0x04
#define CTRL1_SYNC		(1<<27)
#define CTRL1_28		(1<<28)
#define CTRL1_SPWR		(1<<29)
#define CTRL1_RSME		(1<<31)

/* UDFE Interrupt Enable Register */
#define UDFE_IE			0x0c

/* UDFE Interrupt Status Register */
#define UDFE_IS			0x10

#define IRQ_SET_CFG		(1<<7)
#define IRQ_SET_INTF		(1<<8)
#define IRQ_SSPND		(1<<9)
#define IRQ_SOF			(1<<10)
#define IRQ_URST		(1<<11)
#define IRQ_FIFO		(1<<12)
#define IRQ_DMA(x)		(1<<(14+(x)))
#define IRQ_GBL_DMA		(1<<27)
#define IRQ_GLB_EN		(1<<31)

#define IRQ_DMA_EP0OUT		(IRQ_DMA(0))
#define IRQ_DMA_EP0IN		(IRQ_DMA(1))
#define IRQ_DMA_EP1_10 		(IRQ_DMA(2) | IRQ_DMA(3) | IRQ_DMA(4) | IRQ_DMA(5) | \
				IRQ_DMA(6) | IRQ_DMA(7) | IRQ_DMA(8) | IRQ_DMA(9)) | IRQ_DMA(10) | \
				IRQ_DMA(11)
#define IRQ_ALL 		(IRQ_SET_CFG | IRQ_SET_INTF | IRQ_DMA_EP0IN | IRQ_DMA_EP0OUT | \
				IRQ_DMA_EP1_10 | IRQ_GBL_DMA | IRQ_GLB_EN)
/* IRQ_URST |*/ \

/* UDFE Device Controller Programming Control/Status register */
#define UDFE_PRG_CTRL		0x14
#define PRG_CTRL_CSRPRG		(1<<0)
#define PRG_CTRL_DONECSR	(1<<1)
#define PRG_CTRL_SETCSR		(1<<2)

/* Device Descriptor/setup Command register */
#define USBD_DESC_SETUP		0x2000
#define DESC_SETUP_VAL		0x00000100

/* Physical Endpoint Descriptor regsiters */
#define USBD_EP_DESC(x)		(0x2004 + ((x)<<2))
#define DESC_EP_NUM_MASK	(0xf)
#define DESC_EP_IN		(1<<4)
#define DESC_EP_TYPE_MASK	(3<<5)
#define DESC_EP_TYPE_CONTROL	(0<<5)
#define DESC_EP_TYPE_ISO	(1<<5)
#define DESC_EP_TYPE_BULK	(2<<5)
#define DESC_EP_TYPE_INTERRUPT	(3<<5)
#define DESC_EP_CONFIG_MASK	(0xf<<7)
#define DESC_EP_CONFIG_SHIFT	7
#define DESC_EP_IF_MASK		(0xf<<11)
#define DESC_EP_IF_SHIFT	11
#define DESC_EP_IF_ALT_MASK	(0xf<<15)
#define DESC_EP_IF_ALT_SHIFT	15
#define DESC_EP_MAXSIZE(sz)	(((sz) & 0x3ff) << 19)

/* FIFO Interrupt Status registers */
#define FIFO_ISR(x)		(0x3000 + (x) * 0x10)

/* FIFO Interrupt Enable registers */
#define FIFO_IER(x)		(0x3004 + (x) * 0x10)

#define EP_ERROR		(1<<5)
#define EP_NAK			(1<<6)
#define EP_ACK			(1<<7)

/* FIFO Packet Control registers */
#define FIFO_PACKET_CTRL(x)	(0x3080 + (x) * 4)
#define FIFO_PACKET_CTRL_MAX(x)	(((x) & 0x3ff) << 20)

/* FIFO Status and Control registers */
#define FIFO_STS_CTRL(x)	(0x3100 + (x) * 8)
#define STS_CTRL_TIMEOUT_CNT(x)	((x) & 0xf)
#define STS_CTRL_TIMEOUT	(1<<10)
#define STS_CTRL_OVFL		(1<<11)
#define STS_CTRL_M30		(1<<12)
#define STS_CTRL_M31		(1<<13)
#define STS_CTRL_IN		(1<<18)
#define STS_CTRL_CLR		(1<<19)
#define STS_CTRL_TYPE_CONTROL	(0<<20)
#define STS_CTRL_TYPE_ISO	(1<<20)
#define STS_CTRL_TYPE_BULK	(2<<20)
#define STS_CTRL_TYPE_INTERRUPT	(3<<20)
#define STS_CTRL_SEND_STALL	(1<<22)
#define STS_CTRL_STALL_SENT	(1<<23)

#define NS9XXX_ENDPOINTS	11
