/*
 *  linux/include/asm-arm/arch-ns9750/hardware.h
 *
 *  This file contains the hardware definitions of the NS9750.
 *
 *  Copyright (C) 1999 ARM Limited.
 *
 * Copyright (C) 2004 by FS Forth-Systeme GmbH.
 * All rights reserved.
 * Markus Pietrek <mpietrek@fsforth.de>
 * @References: [1] NS9750 Hardware Reference/December 2003
 *              [2] derived from integrator/hardware.h
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef __ASM_ARCH_HARDWARE_H
#define __ASM_ARCH_HARDWARE_H

#include <asm/sizes.h>

/*
 * Where in virtual memory the IO devices (timers, system controllers
 * and so on)
 */
#define IO_BASE			0xF0000000                 // VA of IO
#define IO_SIZE			0x0B000000                 // How much?
#define IO_START		0			   // PA of IO // @TODO

// @TODO do we need to adjust?
/*
 * Similar to above, but for PCI addresses (memory, IO, Config and the
 * V3 chip itself).  WARNING: this has to mirror definitions in platform.h
 */
#define PCI_MEMORY_VADDR        0xe8000000
#define PCI_CONFIG_VADDR        0xec000000
#define PCI_V3_VADDR            0xed000000
#define PCI_IO_VADDR            0xee000000

#define pcibios_assign_all_busses()	1

#define PCIBIOS_MIN_IO		0x6000
#define PCIBIOS_MIN_MEM 	0x00100000

#ifndef __ASSEMBLY__

#include <linux/ioport.h>	/* request_region */

unsigned int ns_sys_clock_freq( void );

#define CPU_CLK_FREQ		(ns_sys_clock_freq()/2)
#define AHB_CLK_FREQ		(ns_sys_clock_freq()/4)
#define BBUS_CLK_FREQ		(ns_sys_clock_freq()/8)


/* request_gpio returns 1 if gpio has been free, otherwise 0. Mapping is done to
 * unused IO_BASE+IO_SIZE memory. */
#if 0
/* don't perform checking */
# define request_gpio( start, length, name ) (1);
# define release_gpio( start, length ) {}
#else
# define request_gpio( start, length, name ) \
	request_region(IO_BASE+IO_SIZE+(start),(length),(name))
# define release_gpio( start, length ) \
	release_region(IO_BASE+IO_SIZE+(start),(length))
#endif

#endif	/* __ASSEMBLY__ */

#endif	/* __ASM_ARCH_HARDWARE_H */

