/***********************************************************************
 *
 * Copyright (C) 2005 by FS Forth-Systeme GmbH.
 * All rights reserved.
 *
 * $Id: ns9360_usb.h,v 1.2 2005-09-06 07:35:00 uzeisberger Exp $
 * @Author:  Jonas Dietsche <jdietsche at fsforth.de>
 * @Descr: Definitions for USB usage
 * @References: [1] NS9360 Hardware Reference Manual/November 2004
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 *
 ***********************************************************************/

#ifndef FS_NS9360_USB_H
#define FS_NS9360_USB_H

#define NS_USB_MODULE_BASE		(0xF9801000) /* virtual address */
#define NS_USB_MODULE_BASE_PA	(0x90801000) /* physikal address */

#define NS_UHFE_BASE			(0xF9800000) /* virtual address */
#define NS_UHFE_BASE_PA		(0x90800000) /* physikal address */

/* the register addresses */

#define NS_USB_IEN			(NS_USB_MODULE_BASE + 0x0C)
#define NS_USB_ISTAT			(NS_USB_MODULE_BASE + 0x10)
#define NS_USB_HCR			(NS_USB_MODULE_BASE)
#define NS_USB_HCR_PORT_SAT	(NS_USB_MODULE_BASE + 0x54)

#define NS_UHFE_CTRL			(NS_UHFE_BASE)
#define NS_UHFE_IEN			(NS_UHFE_BASE + 0xC)
#define NS_UHFE_ISTAT			(NS_UHFE_BASE + 0x10)

/* register bit fields */

#define NS_USB_INT_GBL_EN 		(0x80000000)
#define NS_USB_INT_GBL_DMA 		(0x08000000)
#define NS_USB_INT_DMA_13 		(0x04000000)
#define NS_USB_INT_DMA_12		(0x02000000)
#define NS_USB_INT_DMA_11		(0x01000000)
#define NS_USB_INT_DMA_10		(0x00800000)
#define NS_USB_INT_DMA_9 		(0x00400000)
#define NS_USB_INT_DMA_8 		(0x00200000)
#define NS_USB_INT_DMA_7 		(0x00100000)
#define NS_USB_INT_DMA_6 		(0x00080000)
#define NS_USB_INT_DMA_5 		(0x00040000)
#define NS_USB_INT_DMA_4 		(0x00020000)
#define NS_USB_INT_DMA_3 		(0x00010000)
#define NS_USB_INT_DMA_2 		(0x00008000)
#define NS_USB_INT_DMA_1 		(0x00004000)
#define NS_USB_INT_FIFO	 		(0x00001000)
#define NS_USB_INT_URST	 		(0x00000800)
#define NS_USB_INT_SOF	 		(0x00000400)
#define NS_USB_INT_SUSPEND		(0x00000200)
#define NS_USB_INT_SETINF		(0x00000100)
#define NS_USB_INT_SETCFG		(0x00000080)
#define NS_USB_INT_WAKEUP		(0x00000040)
#define NS_USB_INT_OHCI_IRQ		(0x00000002)

/* the linux interrupt handler can only enable all DMA and all FIFO
 * interrupts. It is not yet possible to disable part of DMA's interrupts. */

#define NS_USB_INT_DMA_ALL		(NS_USB_INT_GBL_DMA | \
					 NS_USB_INT_DMA_13  | \
					 NS_USB_INT_DMA_12  | \
					 NS_USB_INT_DMA_11  | \
					 NS_USB_INT_DMA_10  | \
					 NS_USB_INT_DMA_9   | \
					 NS_USB_INT_DMA_8   | \
					 NS_USB_INT_DMA_7   | \
					 NS_USB_INT_DMA_6   | \
					 NS_USB_INT_DMA_5   | \
					 NS_USB_INT_DMA_4   | \
					 NS_USB_INT_DMA_3   | \
					 NS_USB_INT_DMA_2   | \
					 NS_USB_INT_DMA_1 )

#define NS_USB_INT_FIFO_ALL		(NS_USB_INT_FIFO    | \
					 NS_USB_INT_URST    | \
					 NS_USB_INT_SOF     | \
					 NS_USB_INT_SUSPEND | \
					 NS_USB_INT_SETINF  | \
					 NS_USB_INT_SETCFG  | \
					 NS_USB_INT_WAKEUP  | \
					 NS_USB_INT_OHCI_IRQ)

#define NS_USB_INT_HOST		( NS_USB_INT_OHCI_IRQ )

#define NS_UHFE_CTRL_WAKEUP	(0x00000004)
#define NS_UHFE_CTRL_HSTRST	(0x00000800)

#endif /* FS_NS9360_USB_H */
