/*
 *  linux/include/asm-arm/arch-ns9750/time.h
 *
 * Copyright (C) 2004 by FS Forth-Systeme GmbH.
 * All rights reserved.
 * Markus Pietrek <mpietrek@fsforth.de>
 * @References: [1] NS9750 Hardware Reference/December 2003
 *              [2] derived from arch-integrator/time.h
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***********************************************************************
 *  @History:
 *     2004/09/02 : implemented gettimeoffset
 *     2004/08/20 : ported to kernel 2.6.8
 ***********************************************************************/

#include <linux/ioport.h>	/* request_resource */
#include <linux/interrupt.h>	/* irqreturn_t */

#include <asm/system.h>
#include <asm/mach-types.h>
#include <asm/mach/time.h>	/* timer_tick */
#include <asm/arch/hardware.h>

#include <asm/arch/ns9750_sys.h> /* NS_SYS */
#include <asm/arch/irqs.h>	/* IRQ_TIMER_0 */

#define	CFG_HZ		(CPU_CLK_FREQ/64)
#define TIMER_USED	0

extern unsigned long (*gettimeoffset)( void );
static unsigned long ns9750_gettimeoffset( void );
static irqreturn_t ns9750_timer_interrupt( /*@unused@*/ int irq,
					   void* dev_id,
					   struct pt_regs* regs );
void __init ns_time_init( void );

static struct irqaction ns_timer_irq = {
	.name		= "NS9750 Timer Tick",
	.flags		= SA_INTERRUPT,
	.handler	= ns9750_timer_interrupt
};

static unsigned int ns_timer_reload;
static unsigned int ns_timer_delta_old = 0;
static unsigned int ns_timer_jiffies_old = 0;

/*
 * Returns number of ms since last clock interrupt.  Note that interrupts
 * will have been disabled by do_gettimeoffset()
 */
static unsigned long ns9750_gettimeoffset(void)
{
	unsigned int timer_delta = ns_timer_reload -
		readl( NS_SYS_TIMER_READ( TIMER_USED ) );

	if( ns_timer_jiffies_old == jiffies && ns_timer_delta_old > timer_delta )
		/* we are called by arm/kernel/time.c:do_gettimeofday with
		 * interrupts disabled.
		 * It may happen then that there is a timer reload after
		 * interrupts disable. Therefore jiffies is not incremented.
		 * So while jiffies has not been updated, return the old
		 * time. */
		timer_delta = ns_timer_delta_old;
	else {
		ns_timer_delta_old = timer_delta;
		ns_timer_jiffies_old = jiffies;
	}

	return ( timer_delta * 1000 ) / (CFG_HZ / 1000 );
}

/*
 * IRQ handler for the timer
 */
static irqreturn_t ns9750_timer_interrupt( /*@unused@*/ int irq,
					   void* dev_id,
					   struct pt_regs* regs )
{
	/* acknowledge interrupt */
	rmwl( NS_SYS_TIMER_CTRL( TIMER_USED ), | NS_SYS_TIMER_CTRL_INTC );
	rmwl( NS_SYS_TIMER_CTRL( TIMER_USED ), & ~NS_SYS_TIMER_CTRL_INTC );

	timer_tick( regs );

	return IRQ_HANDLED;
}

static struct resource timer_resource[ 3 ] = {
	{
		.name	= "system timer",
		.start	= (NS_SYS_TIMER_CTRL( TIMER_USED ) ),
		.end	= (NS_SYS_TIMER_CTRL( TIMER_USED ) ) + 3,
	},
	{
		.name	= "system timer",
		.start	= (NS_SYS_TIMER_READ( TIMER_USED ) ),
		.end	= (NS_SYS_TIMER_READ( TIMER_USED ) ) + 3,
	},
	{
		.name	= "system timer",
		.start	= (NS_SYS_TIMER_COUNT( TIMER_USED ) ),
		.end	= (NS_SYS_TIMER_COUNT( TIMER_USED ) ) + 3,
	}
};


/*
 * Set up timer interrupt, and return the current time in seconds.
 */
void __init ns_time_init( void )
{
	request_resource( &iomem_resource, &timer_resource[ 0 ] );
	request_resource( &iomem_resource, &timer_resource[ 1 ] );
	request_resource( &iomem_resource, &timer_resource[ 2 ] );

// 	gettimeoffset = ns9750_gettimeoffset;

	ns_timer_irq.handler = ns9750_timer_interrupt;

	ns_timer_reload = CFG_HZ/HZ; /* a tick every HZ */

	/* U-Boot may leave the timer enabled, reset it */
	writel( 0, NS_SYS_TIMER_CTRL( TIMER_USED ) );

	writel( ns_timer_reload, NS_SYS_TIMER_COUNT( TIMER_USED ) );
	writel( NS_SYS_TIMER_CTRL_TEN |
		NS_SYS_TIMER_CTRL_TLCS_64 |
		NS_SYS_TIMER_CTRL_INTS |
		NS_SYS_TIMER_CTRL_UDS |
		NS_SYS_TIMER_CTRL_TSZ |
		NS_SYS_TIMER_CTRL_REN,
		NS_SYS_TIMER_CTRL( TIMER_USED ) );
	/* @TODO: take RTC time and run do_settimeofday */

	setup_irq( IRQ_TIMER_0 + TIMER_USED, &ns_timer_irq );
}
struct sys_timer ns_timer = {
	.init		= ns_time_init,
	.offset		= ns9750_gettimeoffset
};
