/*
 * linux/include/asm-arm/arch-ns9xxx/dma.h
 *
 * Copyright (C) 2006 by Digi International Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 */
#ifndef __ASM_ARCH_DMA_H
#define __ASM_ARCH_DMA_H

#include <asm/arch/regs-bbus.h>

/* DMA control register bit fields */
#define NS_BBUS_DMA_CTRL_CE		(0x80000000)
#define NS_BBUS_DMA_CTRL_CA		(0x40000000)
#define NS_BBUS_DMA_CTRL_MODE_MA	(0x0C000000)
#define NS_BBUS_DMA_CTRL_MODE_FBW	(0x00000000)
#define NS_BBUS_DMA_CTRL_MODE_FBR	(0x04000000)
#define NS_BBUS_DMA_CTRL_BTE_MA		(0x03000000)
#define NS_BBUS_DMA_CTRL_BTE_1		(0x00000000)
#define NS_BBUS_DMA_CTRL_BTE_2		(0x01000000)
#define NS_BBUS_DMA_CTRL_BTE_4		(0x02000000)
#define NS_BBUS_DMA_CTRL_BDR		(0x00400000)
#define NS_BBUS_DMA_CTRL_RST		(0x00040000)
#define NS_BBUS_DMA_CTRL_STATE_MA	(0x0000FC00)
#define NS_BBUS_DMA_CTRL_INDEX_MA	(0x000003FF)

/* DMA status/interrupt enable register bit fields */
#define NS_BBUS_DMA_STAT_NCIP		(0x80000000)
#define NS_BBUS_DMA_STAT_ECIP		(0x40000000)
#define NS_BBUS_DMA_STAT_NRIP		(0x20000000)
#define NS_BBUS_DMA_STAT_CAIP		(0x10000000)
#define NS_BBUS_DMA_STAT_PCIP		(0x08000000)
#define NS_BBUS_DMA_STAT_NCIE		(0x01000000)
#define NS_BBUS_DMA_STAT_ECIE		(0x00800000)
#define NS_BBUS_DMA_STAT_NRIE		(0x00400000)
#define NS_BBUS_DMA_STAT_CAIE		(0x00200000)
#define NS_BBUS_DMA_STAT_PCIE		(0x00100000)
#define NS_BBUS_DMA_STAT_WRAP		(0x00080000)
#define NS_BBUS_DMA_STAT_IDONE		(0x00040000)
#define NS_BBUS_DMA_STAT_LAST		(0x00020000)
#define NS_BBUS_DMA_STAT_FULL		(0x00010000)
#define NS_BBUS_DMA_STAT_BLEN_MA	(0x0000FFFF)

#define NS_BBUS_DMA_STAT_IP_MA		(NS_BBUS_DMA_STAT_NCIP | \
					 NS_BBUS_DMA_STAT_ECIP | \
					 NS_BBUS_DMA_STAT_NRIP | \
					 NS_BBUS_DMA_STAT_CAIP | \
					 NS_BBUS_DMA_STAT_PCIP)

#define	addr_dma_bdesc(ctrl,ch)		BBUS_DMA_DESCR_VA(ctrl,(ch-1))
#define	addr_dma_ctrl(ctrl,ch)		BBUS_DMA_CTRL_VA(ctrl,(ch-1))
#define	addr_dma_stat(ctrl,ch)		BBUS_DMA_STAT_VA(ctrl,(ch-1))


/* the supported DMA channels per BBUS DMA controller */
typedef enum {
	NS9XXX_BBUS_DMA_CHANNEL_1 = 1,
	NS9XXX_BBUS_DMA_CHANNEL_2,
	NS9XXX_BBUS_DMA_CHANNEL_3,
	NS9XXX_BBUS_DMA_CHANNEL_4,
	NS9XXX_BBUS_DMA_CHANNEL_5,
	NS9XXX_BBUS_DMA_CHANNEL_6,
	NS9XXX_BBUS_DMA_CHANNEL_7,
	NS9XXX_BBUS_DMA_CHANNEL_8,
	NS9XXX_BBUS_DMA_CHANNEL_9,
	NS9XXX_BBUS_DMA_CHANNEL_10,
	NS9XXX_BBUS_DMA_CHANNEL_11,
	NS9XXX_BBUS_DMA_CHANNEL_12,
	NS9XXX_BBUS_DMA_CHANNEL_13,
	NS9XXX_BBUS_DMA_CHANNEL_14,
	NS9XXX_BBUS_DMA_CHANNEL_15,
	NS9XXX_BBUS_DMA_CHANNEL_16
} NS9XXX_dma_ch_t;


typedef struct {
	u32 *src;
	u32 len;	/* 10/11 bits rx/tx */
	u32 *dest;	/* unused */
	union {
		u32 reg;
		struct {
			unsigned status : 16;
			unsigned res : 12;
			unsigned full : 1;
			unsigned last : 1;
			unsigned intr : 1;
			unsigned wrap : 1;
		} bits;
	} s;
} ns_buffer_desc_t;


/* Functions prototypes */
void ns9xxx_disable_dma_ch(int controller, int channel);
void ns9xxx_enable_dma_ch(int controller, int channel);
void ns9xxx_set_options_dma_ch(int controller, int channel,
                               dma_addr_t desc, u32 ctrl_opts, u32 stat_opts);
void ns9xxx_init_dma_desc_list(ns_buffer_desc_t *first_desc, unsigned int list_length,
                               unsigned int enable_irq);

#endif /* ifndef __ASM_ARCH_DMA_H */
