/*
 * include/asm-arm/arch-ns9xxx/hardware.h
 *
 * Copyright (C) 2006 by Digi International Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 */
#ifndef __ASM_ARCH_HARDWARE_H
#define __ASM_ARCH_HARDWARE_H

#include <asm/memory.h>

/* x in [0..3] */
#define NS9XXX_CSxSTAT_PHYS(x)	UL(((x) + 4) << 28)

#define NS9XXX_CS0STAT_LENGTH	UL(0x1000)
#define NS9XXX_CS1STAT_LENGTH	UL(0x1000)
#define NS9XXX_CS2STAT_LENGTH	UL(0x1000)
#define NS9XXX_CS3STAT_LENGTH	UL(0x1000)

#define pcibios_assign_all_busses()     1

#define PCIBIOS_MIN_IO          0x6000
#define PCIBIOS_MIN_MEM         0x00100000
/*
 * NetSilicon NS9xxx internal mapping:
 *
 * physical                <--> virtual
 * 0x40000000 - 0x41ffffff <--> 0xf4000000 - 0xf5ffffff  XXX
 * 0x60000000 - 0x600dffff <--> 0xf6000000 - 0xf60dffff
 * 0x90000000 - 0x909fffff <--> 0xf9000000 - 0xf99fffff
 * 0xa0000000 - 0xa08fffff <--> 0xfa000000 - 0xfa8fffff
 * 0xb0000000 - 0xb0000048 <--> 0xfb000000 - 0xfb000048  Not realy mapped, used by request_gpio
 */
#define io_p2v(x)	(0xf0000000 \
			 + (((x) & 0xf0000000) >> 4) \
			 + ((x) & 0x00ffffff))

#define io_v2p(x)	((((x) & 0x0f000000) << 4) \
			 + ((x) & 0x00ffffff))

#define __REGBIT(bit)		((u32)1 << (bit))
#define __REGBITS(hbit, lbit)	((((u32)1 << ((hbit) - (lbit) + 1)) - 1) << (lbit))
#define __REGVAL(mask, value)	(((value) * ((mask) & (-(mask))) & (mask)))

#ifndef __ASSEMBLY__

#  define __REG(x)	(*((volatile u32 *)io_p2v((x))))
#  define __REG2(x, y)	(*((volatile u32 *)io_p2v((x)) + (y)))
#  define __REG_2(x, y)	(*((volatile u32 *)io_p2v((x)+ (y))))

#  define __REGB(x)	(*((volatile u8 *)io_p2v((x))))
#  define __REGB2(x, y)	(*((volatile u8 *)io_p2v((x)) + (y)))
#  define __REGB_2(x, y)	(*((volatile u8 *)io_p2v((x)+(y))))

#  define REGSET(var, reg, field, value)					\
	((var) = (((var)						\
		   & ~(reg ## _ ## field & 				\
			   ~ reg ## _ ## field ## _ ## value))		\
		  | (reg ## _ ## field ## _ ## value)))

#  define REGSETIM(var, reg, field, value)				\
	((var) = (((var)						\
		   & ~(reg ## _ ## field & ~(__REGVAL(reg ## _ ## field, value))))			\
		  | (__REGVAL(reg ## _ ## field, value))))

#  define REGGET(var, reg, field)					\
	((var & (reg ## _ ## field)) / ((reg ## _ ## field) & (-(reg ## _ ## field))))


/* GPIO helpers */
#define	REQUEST_GPIO_BASE_PM		0xb0000000
#define	REQUEST_GPIO_BASE_VA		io_p2v(REQUEST_GPIO_BASE_PM)

int ns9xxx_request_gpio(unsigned int pin, const char *name);
void ns9xxx_release_gpio(unsigned int pin);
int ns9xxx_gpio_cfgpin(unsigned int pin, unsigned int function);
u32 ns9xxx_gpio_getcfg(unsigned int pin);
int ns9xxx_gpio_invert(unsigned int pin, unsigned int val);
int ns9xxx_gpio_setpin(unsigned int pin, unsigned int val);
unsigned int ns9xxx_gpio_getpin(unsigned int pin);





/* request_gpio returns 1 if gpio has been free, otherwise 0. Mapping is done to
 * unused IO_BASE+IO_SIZE memory. */
#  if 1
/* don't perform checking */
#  else
#    define request_gpio( start, length, name ) \
	request_region(io_p2v(0xB0000000+start),(length),(name))
#    define release_gpio( start, length ) \
	release_region(io_p2v(0xB0000000+start),(length))
#  endif

#else

#  define __REG(x)	io_p2v(x)
#  define __REG2(x, y)	io_p2v((x) + (y))

#  define __REGB(x)	__REG((x))
#  define __REGB2(x, y)	__REG2((x), (y))

#endif

#endif /* ifndef __ASM_ARCH_HARDWARE_H */
