
#include <linux/config.h>
#include <linux/linkage.h>
#include <asm/errno.h>

#include <asm/io.h>
#include <asm/hardware.h>
#include <asm/mach-types.h>
#include <asm/uaccess.h>
#include <asm/mach/map.h>

#include "mhelper.h"


#define IODESC_ENT(x) { (unsigned long)S3C24XX_VA_##x, __phys_to_pfn(S3C2410_PA_##x), S3C24XX_SZ_##x, MT_DEVICE }

#ifdef CONFIG_CPU_S3C2440
extern struct map_desc s3c2440_iodesc[];
extern unsigned long s3c2440_iodesc_size;
#else
extern struct map_desc s3c2410_iodesc[];
extern unsigned long s3c2410_iodesc_size;
#endif

static inline unsigned long
map_io_to_va(unsigned long addr)
{
	int index;

#ifdef CONFIG_CPU_S3C2440
	for (index=0; index<s3c2440_iodesc_size; index++) {
		unsigned long phys = s3c2440_iodesc[index].pfn;
		unsigned long len = s3c2440_iodesc[index].length;

		if ((addr >= phys) & (addr < phys+len))
			break;
	}
	if (index < s3c2440_iodesc_size) {
		unsigned long phys = s3c2440_iodesc[index].pfn;
		unsigned long virt = s3c2440_iodesc[index].virtual;

		return (addr - phys + virt);
	}
#else
	for (index=0; index<s3c2410_iodesc_size; index++) {
		unsigned long phys = s3c2410_iodesc[index].pfn;
		unsigned long len = s3c2410_iodesc[index].length;

		if ((addr >= phys) & (addr < phys+len))
			break;
	}
	if (index < s3c2410_iodesc_size) {
		unsigned long phys = s3c2410_iodesc[index].pfn;
		unsigned long virt = s3c2410_iodesc[index].virtual;

		return (addr - phys + virt);
	}
#endif
	return 0;
}

extern asmlinkage long
sys_mhelper(uint32_t cmd, uint32_t addr, uint32_t value)
{
	unsigned long vaddr = 0;

	if (!(vaddr = map_io_to_va(addr)))
		return -ENXIO;

        switch (cmd) {

	case CMD_READ_B:
		value = __raw_readb(vaddr);
		printk("0x%08x: 0x%02x (%d)\n", addr, value, value);
		break;
	case CMD_READ_W:
		value = __raw_readw(vaddr);
		printk("0x%08x: 0x%04x (%d)\n", addr, value, value);
		break;
	case CMD_READ_L:
		value = __raw_readl(vaddr);
		printk("0x%08x: 0x%08x (%d)\n", addr, value, value);
		break;

	case CMD_WRITE_B:
		__raw_writeb(value, vaddr);
		break;
	case CMD_WRITE_W:
		__raw_writew(value, vaddr);
		break;
	case CMD_WRITE_L:
		__raw_writel(value, vaddr);
		break;

	default:
		return -EINVAL;
		break;
	}

	return 0;
}



